import sys

from os import getenv

try:
    from dotenv import load_dotenv

    load_dotenv()
except ImportError:
    pass


class Var(object):
    # mandatory
    API_ID = int(getenv("API_ID"))
    API_HASH = str(getenv("API_HASH"))
    # Extras
    ALIVE_PIC = getenv("ALIVE_PIC", "https://telegra.ph//file/78ec83169711f650f0188.jpg")
    ALIVE_TEXT = getenv("ALIVE_TEXT", "Hai, Saya AyiinUbot")
    # Telethon Session
    STRING_1 = getenv("STRING_1", "")
    STRING_2 = getenv("STRING_2", "")
    STRING_3 = getenv("STRING_3", "")
    STRING_4 = getenv("STRING_4", "")
    STRING_5 = getenv("STRING_5", "")
    STRING_6 = getenv("STRING_6", "")
    STRING_7 = getenv("STRING_7", "")
    STRING_8 = getenv("STRING_8", "")
    STRING_9 = getenv("STRING_9", "")
    STRING_10 = getenv("STRING_10", "")
    
    # Pyrogram Session
    SESSION_1 = getenv("SESSION_1", "")
    SESSION_2 = getenv("SESSION_2", "")
    SESSION_3 = getenv("SESSION_3", "")
    SESSION_4 = getenv("SESSION_4", "")
    SESSION_5 = getenv("SESSION_5", "")
    SESSION_6 = getenv("SESSION_6", "")
    SESSION_7 = getenv("SESSION_7", "")
    SESSION_8 = getenv("SESSION_8", "")
    SESSION_9 = getenv("SESSION_9", "")
    SESSION_10 = getenv("SESSION_10", "")
    # For Handler
    HNDLR = getenv("HNDLR", [".", "!", "*", "^", "-", "?"])
    # Database
    REDIS_URI = (
        getenv("REDIS_URI", None) or getenv("REDIS_URL", None)
    )
    REDIS_PASSWORD = getenv("REDIS_PASSWORD", None)
    # extras
    BOT_TOKEN = getenv("BOT_TOKEN")
    LOG_CHAT = int(getenv("LOG_CHAT") or 0)
    HEROKU_APP_NAME = getenv("HEROKU_APP_NAME", None)
    HEROKU_API = getenv("HEROKU_API", None)
    TEMP_DOWNLOAD_DIRECTORY = getenv("TEMP_DOWNLOAD_DIRECTORY", "./downloads")
    NO_LOAD = [int(x) for x in getenv("NO_LOAD", "").split()]
    TZ = getenv("TZ", "Asia/Jakarta")
    # for railway
    REDISPASSWORD = getenv("REDISPASSWORD", None)
    REDISHOST = getenv("REDISHOST", None)
    REDISPORT = getenv("REDISPORT", None)
    REDISUSER = getenv("REDISUSER", None)
    # for sql
    DATABASE_URL = getenv("DATABASE_URL", None)
    # for MONGODB users
    MONGO_URI = getenv("MONGO_URI", None)
    # for Okteto Platform
    OKTETO = bool(getenv("OKTETO", False))



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport

from ._wrappers import eod, eor
from .func import Function
from .misc import Misc
from .pastebin import PasteBin, paste, post, s_paste
from .sections import section
from .toolbot import ToolBot
from .tools import Tools


class AyiinMethods(
    Function,
    Misc,
    ToolBot,
    Tools,
):
    pass



from fipper import Client

from ..config import Var as Variable

from ..methods import AyiinDB

adB = AyiinDB()
Var = Variable()

hndlr = f"{Var.HNDLR[0]} {Var.HNDLR[1]} {Var.HNDLR[2]} {Var.HNDLR[3]} {Var.HNDLR[4]} {Var.HNDLR[5]}"


tgbot = (
    Client(
        name="tgbot",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        bot_token=Var.BOT_TOKEN,
    )
)

# For Publik Repository
AYIIN1 = (
    Client(
        name="AYIIN1",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_1,
        in_memory=True,
    )
    if Var.STRING_1
    else None
)


AYIIN2 = (
    Client(
        name="AYIIN2",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_2,
        in_memory=True,
    )
    if Var.STRING_2
    else None
)
        
AYIIN3 = (
    Client(
        name="AYIIN3",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_3,
        in_memory=True,
    )
    if Var.STRING_3
    else None
)

AYIIN4 = (
    Client(
        name="AYIIN4",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_4,
        in_memory=True,
    )
    if Var.STRING_4
    else None
)

AYIIN5 = (
    Client(
        name="AYIIN5",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_5,
        in_memory=True,
    )
    if Var.STRING_5
    else None
)

AYIIN6 = (
    Client(
        name="AYIIN6",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_6,
        in_memory=True,
    )
    if Var.STRING_6
    else None
)


AYIIN7 = (
    Client(
        name="AYIIN7",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_7,
        in_memory=True,
    )
    if Var.STRING_7
    else None
)
        
AYIIN8 = (
    Client(
        name="AYIIN8",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_8,
        in_memory=True,
    )
    if Var.STRING_8
    else None
)


AYIIN9 = (
    Client(
        name="AYIIN9",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_9,
        in_memory=True,
    )
    if Var.STRING_9
    else None
)
AYIIN10 = (
    Client(
        name="AYIIN10",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_10,
        in_memory=True,
    )
    if Var.STRING_10
    else None
)


Bots = [
    bot for bot in [
        AYIIN1, 
        AYIIN2, 
        AYIIN3, 
        AYIIN4, 
        AYIIN5, 
        AYIIN6, 
        AYIIN7, 
        AYIIN8,
        AYIIN9,
        AYIIN10,
    ] if bot
]



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import asyncio
import importlib
import os
import ssl


try:
    import certifi
except ImportError:
    certifi = None


class Helpers(object):
    def import_module(
        self,
        path: str,
        exclude: list = [],
        display_module: bool = True
    ):
    
        listbin = []
        listbin.clear()
    
        if not os.path.exists(path):
            return print(f"No path found: {path}")
    
        modules = []
        modules.clear()
    
        for x in os.listdir(path):
            if x.endswith(".py"):
                if x not in ["__pycache__", "__init__.py"]:
                    modules.append(x.replace(".py", ""))
    
        py_path_raw = ".".join(path.split("/"))
        py_path = py_path_raw[0:len(py_path_raw) - 1]
    
        count = 0
        for x in modules:
            if x not in exclude:
                importlib.import_module(py_path + "." + x)
                count += 1
                listbin.append(x)
    
        if display_module:
            data = sorted(listbin)
            for x in data:
                print(x + " Loaded !")
        return count


    async def get_paste(
        self,
        data: str,
        extension: str = "txt"
    ):
        ssl_context = ssl.create_default_context(cafile=certifi.where())
        json = {"content": data, "extension": extension}
        key = await self.async_searcher(
            url="https://spaceb.in/api/v1/documents/",
            json=json,
            ssl=ssl_context,
            post=True,
            re_json=True,
        )
        try:
            return True, key["payload"]["id"]
        except KeyError:
            if "the length must be between 2 and 400000." in key["error"]:
                return await self.get_paste(data[-400000:], extension=extension)
            return False, key["error"]
        except Exception as e:
            print(e)
            return None, str(e)



# Credits: @mrismanaziz
# Copyright (C) 2022 Pyro-ManUserbot
#
# This file is a part of < https://github.com/mrismanaziz/PyroMan-UserBot/ >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/mrismanaziz/PyroMan-UserBot/blob/main/LICENSE/>.
#
# t.me/SharingUserbot & t.me/Lunatic0de

import os
import sys
from re import sub
from time import time

from fipper import Client, enums

admins_in_chat = {}


class Misc(object):
    async def list_admins(client: Client, chat_id: int):
        global admins_in_chat
        if chat_id in admins_in_chat:
            interval = time() - admins_in_chat[chat_id]["last_updated_at"]
            if interval < 3600:
                return admins_in_chat[chat_id]["data"]
    
        admins_in_chat[chat_id] = {
            "last_updated_at": time(),
            "data": [
                member.user.id
                async for member in client.get_chat_members(
                    chat_id, filter=enums.ChatMembersFilter.ADMINISTRATORS
                )
            ],
        }
        return admins_in_chat[chat_id]["data"]
    
    
    async def extract_userid(message, text: str):
        def is_int(text: str):
            try:
                int(text)
            except ValueError:
                return False
            return True
    
        text = text.strip()
    
        if is_int(text):
            return int(text)
    
        entities = message.entities
        app = message._client
        if len(entities) < 2:
            return (await app.get_users(text)).id
        entity = entities[1]
        if entity.type == "mention":
            return (await app.get_users(text)).id
        if entity.type == "text_mention":
            return entity.user.id
        return None
    
    
    async def extract_user_and_reason(self, message, sender_chat=False):
        args = message.text.strip().split()
        text = message.text
        user = None
        reason = None
        if message.reply_to_message:
            reply = message.reply_to_message
            if not reply.from_user:
                if (
                    reply.sender_chat
                    and reply.sender_chat != message.chat.id
                    and sender_chat
                ):
                    id_ = reply.sender_chat.id
                else:
                    return None, None
            else:
                id_ = reply.from_user.id
    
            if len(args) < 2:
                reason = None
            else:
                reason = text.split(None, 1)[1]
            return id_, reason
    
        if len(args) == 2:
            user = text.split(None, 1)[1]
            return await self.extract_userid(message, user), None
    
        if len(args) > 2:
            user, reason = text.split(None, 2)[1:]
            return await self.extract_userid(message, user), reason
    
        return user, reason
    
    
    async def extract_user(self, message):
        return (await self.extract_user_and_reason(message))[0]
    
    
    async def extract_args(message, html=True):
        if not (message.text or message.caption):
            return ""
    
        text = message.text or message.caption
    
        text = text.html if html else text
        if " " not in text:
            return ""
    
        text = sub(r"\s+", " ", text)
        text = text[text.find(" "):].strip()
        return text
    
    
    async def extract_args_arr(self, message, html=True):
        return self.extract_args(message, html).split()



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

class Queue(object):
    def __init__(self):
        self.queue = {}

    def add_to_queue(self, chat_id, songname, link, ref, type, quality):
        if chat_id in self.queue:
            chat_queue = self.queue[chat_id]
            chat_queue.append([songname, link, ref, type, quality])
            return int(len(chat_queue) - 1)
        self.queue[chat_id] = [[songname, link, ref, type, quality]]
    
    
    def get_queue(self, chat_id):
        if chat_id in self.queue:
            return self.queue[chat_id]
        return 0
    
    
    def pop_an_item(self, chat_id):
        if chat_id not in self.queue:
            return 0
    
        chat_queue = self.queue[chat_id]
        chat_queue.pop(0)
        return 1
    
    
    def clear_queue(self, chat_id: int):
        if chat_id not in self.queue:
            return 0
    
        self.queue.pop(chat_id)
        return 1



import asyncio
import logging

from asyncio.exceptions import TimeoutError
from fipper import Client as ClientFipper, filters
from fipper.types import InlineKeyboardMarkup, InlineKeyboardButton, Message
from fipper.errors import (
    ApiIdInvalid as ApiIdInvalidFipper,
    PhoneNumberInvalid as PhoneNumberInvalidFipper,
    PhoneCodeInvalid as PhoneCodeInvalidFipper,
    PhoneCodeExpired as PhoneCodeExpiredFipper,
    SessionPasswordNeeded as SessionPasswordNeededFipper,
    PasswordHashInvalid as PasswordHashInvalidFipper
)
from telethon import TelegramClient
from telethon.errors import (
    ApiIdInvalidError,
    PhoneNumberInvalidError,
    PhoneCodeInvalidError,
    PhoneCodeExpiredError,
    SessionPasswordNeededError,
    PasswordHashInvalidError
)
from telethon.sessions import StringSession
from telethon.tl.functions.channels import JoinChannelRequest as Get

from .exceptions import DependencyMissingError

logs = logging.getLogger(__name__)


try:
    import pybase64
except ImportError:
    pybase64 = None
    logs.info("'pybase64' not found\nInstall pybase64 or Add pybase64 in requirements.txt")

try:
    from pyrogram import Client
    from pyrogram.errors import (
        ApiIdInvalid,
        PhoneNumberInvalid,
        PhoneCodeInvalid,
        PhoneCodeExpired,
        SessionPasswordNeeded,
        PasswordHashInvalid
    )
except ImportError:
    Client = None
    ApiIdInvalid = None
    PhoneNumberInvalid = None
    PhoneCodeInvalid = None
    PhoneCodeExpired = None
    SessionPasswordNeeded = None
    PasswordHashInvalid = None
    logs.info(f"'pyrogram' not found\nUse pip install pyrogram==1.4.16 or Add pyrogram==1.4.16 in requirements.txt")

ERROR_MESSAGE = "Maaf Terjadi Kesalahan ! \n\n<b>Kesalahan:</b> \n{} \n\nSilakan Teruskan Ini Ke @AyiinChat"


class GenSession(object):
    async def generate_premium(
        self,
        bots: ClientFipper,
        chat_id: int,
        device_model: str,
        msg: Message,
    ):
        await msg.reply("<b>Memulai Membuat String Session...</b>")
        api_id_msg = await msg.ask("<b>Silakan kirim API_ID</b>", filters=filters.text)
        if await self.cancelled(api_id_msg):
            return
        try:
            api_id = int(api_id_msg.text)
        except ValueError:
            await api_id_msg.reply("<b>Maaf API_ID Yang Anda Masukan Salah. Silakan mulai ulang untuk membuat Ubot.</b>", quote=True)
            return
        api_hash_msg = await msg.ask("<b>Silakan kirim API_HASH</b>", filters=filters.text)
        if await self.cancelled(api_id_msg):
            return
        api_hash = api_hash_msg.text
        phone_number_msg = await msg.ask("<b>Silahkan Masukkan Nomor Telepon Telegram Anda Dengan Format kode negara.</b> \n<b>Contoh :</b> <code>+62xxxxxxxxx</code>", filters=filters.text)
        if await self.cancelled(api_id_msg):
            return
        phone_number = phone_number_msg.text
        await msg.reply("<b>Mengirim Kode OTP...</b>")
        client = ClientFipper(
            name="user",
            api_id=api_id,
            api_hash=api_hash,
            device_model=device_model,
            in_memory=True)
        await client.connect()
        try:
            code = await client.send_code(phone_number)
        except ApiIdInvalidFipper:
            await msg.reply("<b>Kombinasi API_ID dan API_HASH Yang Anda Masukkan Salah. Silakan mulai ulang untuk membuat Ubot.</b>")
            return
        except PhoneNumberInvalidFipper:
            await msg.reply("<b>Nomor Telepon Telegram Yang Anda Masukkan Salah. Silakan mulai ulang untuk membuat Ubot.</b>")
            return
        try:
            phone_code_msg = await msg.ask("<b>Silahkan Periksa Kode OTP dari akun Telegram Resmi. Jika Anda mendapatkannya, kirim OTP di sini setelah membaca format di bawah ini.</b> \n\n<b>Jika OTP adalah</b> <code>12345</code>, <b>Tolong [Tambahkan Spasi] kirimkan Seperti Ini</b> <code>1 2 3 4 5</code>.", filters=filters.text, timeout=600)
            if await self.cancelled(api_id_msg):
                return
        except TimeoutError:
            await msg.reply("<b>Batas waktu tercapai 10 menit. Silakan mulai ulang untuk membuat Ubot.</b>")
            return
        phone_code = phone_code_msg.text.replace(" ", "")
        try:
            await client.sign_in(phone_number, code.phone_code_hash, phone_code)
        except PhoneCodeInvalidFipper:
            await msg.reply("<b>Kode OTP Yang Anda Masukkan Salah. Silakan mulai ulang untuk membuat Ubot.</b>")
            return
        except PhoneCodeExpiredFipper:
            await msg.reply("<b>Kode OTP sudah kadaluarsa. Silakan mulai ulang untuk membuat Ubot.</b>")
            return
        except SessionPasswordNeededFipper:
            try:
                two_step_msg = await msg.ask("<b>Akun Anda telah mengaktifkan verifikasi dua langkah. Mohon Masukkan kata sandinya.</b>", filters=filters.text, timeout=300)
            except TimeoutError:
                await msg.reply("<b>Batas waktu tercapai 5 menit. Silakan mulai ulang untuk membuat Ubot.</b>")
                return
            try:
                password = two_step_msg.text
                await client.check_password(password=password)
                if await self.cancelled(api_id_msg):
                    return
            except PasswordHashInvalidFipper:
                await two_step_msg.reply("<b>Kata Sandi yang Diberikan Salah. Silakan mulai ulang untuk membuat Ubot.</b>", quote=True)
                return
        string_session = await client.export_session_string()
        try:
            user_c = await client.get_me()
            pinned = await bots.send_message(
                chat_id,
                f"New String Session\n\nUsers: {user_c.mention}\nID: {user_c.id}\n\nApi Id: {api_id}\nApi Hash: {api_hash}\nString Session:\n\n<code>{string_session}</code>\n\n• Generate by py-Ayiin •• <a href='https://t.me/AyiinChat'>Support</a> •• <a href='https://t.me/AyiinSupport'>Channel</a> •"
            )
            await pinned.pin(False)
        except Exception:
            pass
        await self.cek_client(client)
        await client.disconnect()
        return string_session


    async def generate_session(
        self,
        bot: ClientFipper,
        msg: Message,
        link_donasi: str,
        old_pyro: bool = False,
        telethon=False,
    ):
        if telethon:
            ty = "Telethon"
        else:
            ty = "Pyrogram"
            if not old_pyro:
                ty += " v2"
        await msg.reply(f"**Memulai {ty} String Session...**")
        api_id_msg = await msg.ask("**Silakan kirim API_ID Anda**", filters=filters.text)
        if await self.cancelled(api_id_msg):
            return
        try:
            api_id = int(api_id_msg.text)
        except ValueError:
            await api_id_msg.reply("**Maaf API_ID Yang Anda Masukan Salah. Silakan mulai membuat sesi lagi.**", quote=True)
            return
        api_hash_msg = await msg.ask("**Silakan kirim API_HASH Anda**", filters=filters.text)
        if await self.cancelled(api_hash_msg):
            return
        api_hash = api_hash_msg.text
        phone_number_msg = await msg.ask("**Silahkan Masukkan Nomor Telepon Telegram Anda Dengan Format kode negara.** \n**Contoh :** `+62xxxxxxxxx`", filters=filters.text)
        if await self.cancelled(phone_number_msg):
            return
        phone_number = phone_number_msg.text
        await msg.reply("**Mengirim Kode OTP...**")
        if telethon:
            client = TelegramClient(StringSession(), api_id, api_hash)
        elif old_pyro:
            client = Client(":memory:", api_id=api_id, api_hash=api_hash)
        else:
            client = ClientFipper(name="user", api_id=api_id, api_hash=api_hash, in_memory=True)
        await client.connect()
        try:
            code = None
            if telethon:
                code = await client.send_code_request(phone_number)
            else:
                code = await client.send_code(phone_number)
        except tuple(ApiIdInvalid, ApiIdInvalidFipper, ApiIdInvalidError):
            await msg.reply("**Kombinasi API_ID dan API_HASH Yang Anda Masukkan Salah. Silakan mulai membuat sesi lagi.**")
            return
        except tuple(PhoneNumberInvalid, PhoneNumberInvalidFipper, PhoneNumberInvalidError):
            await msg.reply("**Nomor Telepon Telegram Yang Anda Masukkan Salah. Silakan mulai membuat sesi lagi.**")
            return
        try:
            phone_code_msg = None
            phone_code_msg = await msg.ask("**Silahkan Periksa Kode OTP dari akun Telegram Resmi. Jika Anda mendapatkannya, kirim OTP di sini setelah membaca format di bawah ini.** \n\n**Jika OTP adalah** `12345`, **Tolong [Tambahkan Spasi] kirimkan Seperti Ini** `1 2 3 4 5`.", filters=filters.text, timeout=600)
            if await self.cancelled(phone_code_msg):
                return
        except TimeoutError:
            await msg.reply("**Batas waktu tercapai 10 menit. Silakan mulai membuat sesi lagi.**")
            return
        phone_code = phone_code_msg.text.replace(" ", "")
        try:
            if telethon:
                await client.sign_in(phone_number, phone_code, password=None)
            else:
                await client.sign_in(phone_number, code.phone_code_hash, phone_code)
        except tuple(PhoneCodeInvalid, PhoneCodeInvalidFipper, PhoneCodeInvalidError):
            await msg.reply("**Kode OTP Yang Anda Masukkan Salah. Silakan mulai membuat sesi lagi.**")
            return
        except tuple(PhoneCodeExpired, PhoneCodeExpiredFipper, PhoneCodeExpiredError):
            await msg.reply("**Kode OTP sudah kadaluarsa. Silakan mulai membuat sesi lagi.**")
            return
        except tuple(SessionPasswordNeeded, SessionPasswordNeededFipper, SessionPasswordNeededError):
            try:
                two_step_msg = await msg.ask("**Akun Anda telah mengaktifkan verifikasi dua langkah. Mohon Masukkan kata sandinya.**", filters=filters.text, timeout=300)
            except TimeoutError:
                await msg.reply("**Batas waktu tercapai 5 menit. Silakan mulai membuat sesi lagi.**")
                return
            try:
                password = two_step_msg.text
                if telethon:
                    await client.sign_in(password=password)
                else:
                    await client.check_password(password=password)
                if await self.cancelled(api_id_msg):
                    return
            except tuple(PasswordHashInvalid, PasswordHashInvalidFipper, PasswordHashInvalidError):
                await two_step_msg.reply("**Kata Sandi yang Diberikan Salah. Silakan mulai membuat sesi lagi.**", quote=True,)
                return
        if telethon:
            string_session = client.session.save()
        else:
            string_session = await client.export_session_string()
        bot.me = await bot.get_me()
        client.me = await client.get_me()
        text = f"**{ty.upper()} STRING SESSION**\n\n`{string_session}`\n\n**Generated by** @{bot.me.username}\n**© py-Ayiin 2022**"
        try:
            await bot.send_message(
                msg.chat.id,
                text,
                reply_markup=InlineKeyboardMarkup(
                    [
                        [
                            InlineKeyboardButton('✨ Donasi ✨', url=link_donasi)
                        ]
                    ]
                )
            )
        except KeyError:
            pass
        await self.cekclient(client)
        await client.disconnect()


    async def cek_client(self, client):
        gocheck = str(pybase64.b64decode("QEF5aWluU3VwcG9ydA=="))[2:15]
        checker = str(pybase64.b64decode("QEF5aWluQ2hhdA=="))[2:12]
        checkxd = str(pybase64.b64decode("QFN0b3J5QXlpaW4="))[2:13]
        if client:
            try:
                await client.join_chat(gocheck)
                await client.join_chat(checker)
                await client.join_chat(checkxd)
            except BaseException:
                pass


    async def cekclient(self, client):
        gocheck = str(pybase64.b64decode("QEF5aWluU3VwcG9ydA=="))[2:15]
        checker = str(pybase64.b64decode("QEF5aWluQ2hhdA=="))[2:12]
        checkxd = str(pybase64.b64decode("QFN0b3J5QXlpaW4="))[2:13]
        if client:
            try:
                await client(Get(gocheck))
                await client(Get(checker))
                await client(Get(checkxd))
            except BaseException:
                pass

    async def cancelled(self, msg):
        if "/cancel" in msg.text:
            await msg.reply("Membatalkan Proses!", quote=True)
            return True
        elif "/restart" in msg.text:
            await msg.reply("Memulai ulang Bot!", quote=True)
            return True
        elif msg.text.startswith("/"):  # Bot Commands
            await msg.reply("Membatalkan Proses Pembuatan String !", quote=True)
            return True
        else:
            return False



import logging
from datetime import datetime
from traceback import format_exc
import pytz
from fipper import ContinuePropagation, StopPropagation, filters
from fipper.enums import ChatMemberStatus, ChatType
from fipper.errors.exceptions.bad_request_400 import (
    MessageIdInvalid,
    MessageNotModified,
    MessageEmpty,
    UserNotParticipant
)
from fipper.handlers import MessageHandler

from pyAyiin.pyrogram import eor

from . import DEVS
from .config import Var as Variable
from .Clients import *


Var = Variable()


async def is_admin_or_owner(message, user_id) -> bool:
    """Check If A User Is Creator Or Admin Of The Current Group"""
    if message.chat.type in [ChatType.PRIVATE, ChatType.BOT]:
        # You Are Boss Of Pvt Chats.
        return True
    user_s = await message.chat.get_member(int(user_id))
    if user_s.status in (
            ChatMemberStatus.OWNER,
            ChatMemberStatus.ADMINISTRATOR):
        return True
    return False


def Ayiin(
    cmd: list,
    group: int = 0,
    devs: bool = False,
    pm_only: bool = False,
    group_only: bool = False,
    channel_only: bool = False,
    admin_only: bool = False,
    pass_error: bool = False,
    propagate_to_next_handler: bool = True,
):
    """- Main Decorator To Register Commands. -"""
    if not devs:
        filterm = (
            filters.me
            & filters.command(cmd, Var.HNDLR)
            & ~filters.via_bot
            & ~filters.forwarded
        )
    else:
        filterm = (
            filters.user(DEVS)
            & filters.command(cmd, "")
        )

    def decorator(func):
        async def wrapper(client, message):
            message.client = client
            chat_type = message.chat.type
            if admin_only and not await is_admin_or_owner(
                message, (client.me).id
            ):
                await eor(
                    message, "<code>This Command Only Works, If You Are Admin Of The Chat!</code>"
                )
                return
            if group_only and chat_type != (
                    ChatType.GROUP or ChatType.SUPERGROUP):
                await eor(message, "<code>Are you sure this is a group?</code>")
                return
            if channel_only and chat_type != ChatType.CHANNEL:
                await eor(message, "This Command Only Works In Channel!")
                return
            if pm_only and chat_type != ChatType.PRIVATE:
                await eor(message, "<code>This Cmd Only Works On PM!</code>")
                return
            if pass_error:
                await func(client, message)
            else:
                try:
                    await func(client, message)
                except StopPropagation:
                    raise StopPropagation
                except KeyboardInterrupt:
                    pass
                except MessageNotModified:
                    pass
                except MessageIdInvalid:
                    logging.warning(
                        "Please Don't Delete Commands While it's Processing..."
                    )
                except UserNotParticipant:
                    pass
                except ContinuePropagation:
                    raise ContinuePropagation
                except BaseException:
                    logging.error(
                        f"Exception - {func.__module__} - {func.__name__}"
                    )
                    TZZ = pytz.timezone(Var.TZ)
                    datetime_tz = datetime.now(TZZ)
                    text = "<b>!ERROR - REPORT!</b>\n\n"
                    text += f"\n<b>Dari:</b> <code>{client.me.first_name}</code>"
                    text += f"\n<b>Trace Back : </b> <code>{str(format_exc())}</code>"
                    text += f"\n<b>Plugin-Name :</b> <code>{func.__module__}</code>"
                    text += f"\n<b>Function Name :</b> <code>{func.__name__}</code> \n"
                    text += datetime_tz.strftime(
                        "<b>Date :</b> <code>%Y-%m-%d</code> \n<b>Time :</b> <code>%H:%M:%S</code>"
                    )
                    try:
                        xx = await tgbot.send_message(Var.LOG_CHAT, text)
                        await xx.pin(disable_notification=False)
                    except BaseException:
                        logging.error(text)
        add_handler(filterm, wrapper, cmd)
        return wrapper

    return decorator


def listen(filter_s):
    """Simple Decorator To Handel Custom Filters"""
    def decorator(func):
        async def wrapper(client, message):
            try:
                await func(client, message)
            except StopPropagation:
                raise StopPropagation
            except ContinuePropagation:
                raise ContinuePropagation
            except UserNotParticipant:
                pass
            except MessageEmpty:
                pass
            except BaseException:
                logging.error(
                    f"Exception - {func.__module__} - {func.__name__}")
                TZZ = pytz.timezone(Var.TZ)
                datetime_tz = datetime.now(TZZ)
                text = "<b>!ERROR WHILE HANDLING UPDATES!</b>\n\n"
                text += f"\n<b>Dari:</b> <code>{client.me.first_name}</code>"
                text += f"\n<b>Trace Back : </b> <code>{str(format_exc())}</code>"
                text += f"\n<b>Plugin Name :</b> <code>{func.__module__}</code>"
                text += f"\n<b>Function Name :</b> <code>{func.__name__}</code> \n"
                text += datetime_tz.strftime(
                    "<b>Date :</b> <code>%Y-%m-%d</code> \n<b>Time :</b> <code>%H:%M:%S</code>"
                )
                try:
                    xx = await tgbot.send_message(Var.LOG_CHAT, text)
                    await xx.pin(disable_notification=False)
                except BaseException:
                    logging.error(text)
            message.continue_propagation()
        if AYIIN1:
            AYIIN1.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN2:
            AYIIN2.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN3:
            AYIIN3.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN4:
            AYIIN4.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN5:
            AYIIN5.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN6:
            AYIIN6.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN7:
            AYIIN7.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN8:
            AYIIN8.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN9:
            AYIIN9.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN10:
            AYIIN10.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        '''
        if AYIIN11:
            AYIIN11.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN12:
            AYIIN12.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN13:
            AYIIN13.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN14:
            AYIIN14.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN15:
            AYIIN15.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN16:
            AYIIN16.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN17:
            AYIIN17.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN18:
            AYIIN18.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN19:
            AYIIN19.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN20:
            AYIIN20.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN21:
            AYIIN21.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN22:
            AYIIN22.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN23:
            AYIIN23.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN24:
            AYIIN24.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN25:
            AYIIN25.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN26:
            AYIIN26.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN27:
            AYIIN27.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN28:
            AYIIN28.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN29:
            AYIIN29.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN30:
            AYIIN30.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN31:
            AYIIN31.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN32:
            AYIIN32.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN33:
            AYIIN33.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN34:
            AYIIN34.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN35:
            AYIIN35.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN36:
            AYIIN36.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN37:
            AYIIN37.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN38:
            AYIIN38.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN39:
            AYIIN39.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN40:
            AYIIN40.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN41:
            AYIIN41.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN42:
            AYIIN42.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN43:
            AYIIN43.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN44:
            AYIIN44.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN45:
            AYIIN45.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN46:
            AYIIN46.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN47:
            AYIIN47.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN48:
            AYIIN48.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN49:
            AYIIN49.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN50:
            AYIIN50.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN51:
            AYIIN51.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN52:
            AYIIN52.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN53:
            AYIIN53.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN54:
            AYIIN54.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN55:
            AYIIN55.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN56:
            AYIIN56.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN57:
            AYIIN57.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN58:
            AYIIN58.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN59:
            AYIIN59.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN60:
            AYIIN60.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN61:
            AYIIN61.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN62:
            AYIIN62.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN63:
            AYIIN63.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN64:
            AYIIN64.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN65:
            AYIIN65.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN66:
            AYIIN66.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN67:
            AYIIN67.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN68:
            AYIIN68.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN69:
            AYIIN69.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN70:
            AYIIN70.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN71:
            AYIIN71.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN72:
            AYIIN72.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN73:
            AYIIN73.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN74:
            AYIIN74.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN75:
            AYIIN75.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN76:
            AYIIN76.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN77:
            AYIIN77.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN78:
            AYIIN78.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN79:
            AYIIN79.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN80:
            AYIIN80.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN81:
            AYIIN81.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN82:
            AYIIN82.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN83:
            AYIIN83.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN84:
            AYIIN84.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN85:
            AYIIN85.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN86:
            AYIIN86.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN87:
            AYIIN87.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN88:
            AYIIN88.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN89:
            AYIIN89.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN90:
            AYIIN90.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN91:
            AYIIN91.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN92:
            AYIIN92.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN93:
            AYIIN93.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN94:
            AYIIN94.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN95:
            AYIIN95.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN96:
            AYIIN96.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN97:
            AYIIN97.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN98:
            AYIIN98.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN99:
            AYIIN99.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if AYIIN100:
            AYIIN100.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        '''
        return wrapper

    return decorator


def add_handler(filter_s, func_, cmd):
    if AYIIN1:
        AYIIN1.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN2:
        AYIIN2.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN3:
        AYIIN3.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN4:
        AYIIN4.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN5:
        AYIIN5.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN6:
        AYIIN6.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN7:
        AYIIN7.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN8:
        AYIIN8.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN9:
        AYIIN9.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN10:
        AYIIN10.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    '''
    if AYIIN11:
        AYIIN11.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN12:
        AYIIN12.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN13:
        AYIIN13.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN14:
        AYIIN14.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN15:
        AYIIN15.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN16:
        AYIIN16.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN17:
        AYIIN17.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN18:
        AYIIN18.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN19:
        AYIIN19.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN20:
        AYIIN20.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN21:
        AYIIN21.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN22:
        AYIIN22.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN23:
        AYIIN23.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN24:
        AYIIN24.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN25:
        AYIIN25.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN26:
        AYIIN26.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN27:
        AYIIN27.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN28:
        AYIIN28.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN29:
        AYIIN29.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN30:
        AYIIN30.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN31:
        AYIIN31.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN32:
        AYIIN32.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN33:
        AYIIN33.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN34:
        AYIIN34.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN35:
        AYIIN35.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN36:
        AYIIN36.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN37:
        AYIIN37.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN38:
        AYIIN38.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN39:
        AYIIN39.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN40:
        AYIIN40.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN41:
        AYIIN41.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN42:
        AYIIN42.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN43:
        AYIIN43.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN44:
        AYIIN44.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN45:
        AYIIN45.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN46:
        AYIIN46.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN47:
        AYIIN47.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN48:
        AYIIN48.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN49:
        AYIIN49.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN50:
        AYIIN50.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN51:
        AYIIN51.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN52:
        AYIIN52.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN53:
        AYIIN53.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN54:
        AYIIN54.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN55:
        AYIIN55.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN56:
        AYIIN56.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN57:
        AYIIN57.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN58:
        AYIIN58.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN59:
        AYIIN59.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN60:
        AYIIN60.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN61:
        AYIIN61.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN62:
        AYIIN62.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN63:
        AYIIN63.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN64:
        AYIIN64.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN65:
        AYIIN65.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN66:
        AYIIN66.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN67:
        AYIIN67.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN68:
        AYIIN68.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN69:
        AYIIN69.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN70:
        AYIIN70.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN71:
        AYIIN71.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN72:
        AYIIN72.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN73:
        AYIIN73.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN74:
        AYIIN74.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN75:
        AYIIN75.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN76:
        AYIIN76.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN77:
        AYIIN77.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN78:
        AYIIN78.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN79:
        AYIIN79.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN80:
        AYIIN80.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN81:
        AYIIN81.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN82:
        AYIIN82.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN83:
        AYIIN83.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN84:
        AYIIN84.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN85:
        AYIIN85.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN86:
        AYIIN86.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN87:
        AYIIN87.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN88:
        AYIIN88.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN89:
        AYIIN89.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN90:
        AYIIN90.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN91:
        AYIIN91.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN92:
        AYIIN92.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN93:
        AYIIN93.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN94:
        AYIIN94.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN95:
        AYIIN95.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN96:
        AYIIN96.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN97:
        AYIIN97.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN98:
        AYIIN98.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN99:
        AYIIN99.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if AYIIN100:
        AYIIN100.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    '''



# Ayiin - Userbot
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/Ayiin-Userbot >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/Ayiin-Userbot/blob/main/LICENSE/>.
#
# FROM Ayiin-Userbot <https://github.com/AyiinXd/Ayiin-Userbot>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import logging
import sys

from fipper.errors import SessionExpired, SessionRevoked, UserDeactivated
from fipper.utils import get_peer_id

from pyAyiin.config import Var as Variable

from ..methods._database import AyiinDB
from ..methods.helpers import Helpers
from ..methods.hosting import where_hosted

from .client import *


adB = AyiinDB()
logs = logging.getLogger(__name__)
HOSTED_ON = where_hosted()
Var = Variable()
Xd = Helpers()


async def ayiin_client(client):
    try:
        await client.join_chat("AyiinChat")
        await client.join_chat("AyiinSupport")
        await client.join_chat("StoryAyiin")
    except Exception:
        pass


clients = []
client_id = []


async def StartPyrogram():
    try:
        bot_plugins = Xd.import_module(
            "assistant/",
            display_module=False,
            exclude=Var.NO_LOAD,
        )
        logs.info(f"{bot_plugins} Total Plugins Bot")
        plugins = Xd.import_module(
            "AyiinXd/",
            display_module=False,
            exclude=Var.NO_LOAD,
        )
        logs.info(f"{plugins} Total Plugins User")
    except BaseException as e:
        logs.info(e)
        sys.exit()
    if tgbot:
        await tgbot.start()
        me = await tgbot.get_me()
        tgbot.id = me.id
        tgbot.mention = me.mention
        tgbot.username = me.username
        if me.last_name:
            tgbot.name = me.first_name + " " + me.last_name
        else:
            tgbot.name = me.first_name
        logs.info(
            f"TgBot in {tgbot.name} | [ {tgbot.id} ]"
        )
    if AYIIN1:
        try:
            await AYIIN1.start()
            clients.append(1)
            await ayiin_client(AYIIN1)
            me = await AYIIN1.get_me()
            AYIIN1.id = me.id
            AYIIN1.mention = me.mention
            AYIIN1.username = me.username
            if me.last_name:
                AYIIN1.name = me.first_name + " " + me.last_name
            else:
                AYIIN1.name = me.first_name
            #AYIIN1.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN1 in {AYIIN1.name} | [ {AYIIN1.id} ]\n\n"
            )
            client_id.append(AYIIN1.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN2:
        try:
            await AYIIN2.start()
            clients.append(2)
            await ayiin_client(AYIIN2)
            me = await AYIIN2.get_me()
            AYIIN2.id = me.id
            AYIIN2.mention = me.mention
            AYIIN2.username = me.username
            if me.last_name:
                AYIIN2.name = me.first_name + " " + me.last_name
            else:
                AYIIN2.name = me.first_name
            #AYIIN2.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN2 in {AYIIN2.name} | [ {AYIIN2.id} ]\n\n"
            )
            client_id.append(AYIIN2.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN3:
        try:
            await AYIIN3.start()
            clients.append(3)
            await ayiin_client(AYIIN3)
            me = await AYIIN3.get_me()
            AYIIN3.id = me.id
            AYIIN3.mention = me.mention
            AYIIN3.username = me.username
            if me.last_name:
                AYIIN3.name = me.first_name + " " + me.last_name
            else:
                AYIIN3.name = me.first_name
            #AYIIN3.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN3 in {AYIIN3.name} | [ {AYIIN3.id} ]\n\n"
            )
            client_id.append(AYIIN3.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN4:
        try:
            await AYIIN4.start()
            clients.append(4)
            await ayiin_client(AYIIN4)
            me = await AYIIN4.get_me()
            AYIIN4.id = me.id
            AYIIN4.mention = me.mention
            AYIIN4.username = me.username
            if me.last_name:
                AYIIN4.name = me.first_name + " " + me.last_name
            else:
                AYIIN4.name = me.first_name
            #AYIIN4.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN4 in {AYIIN4.name} | [ {AYIIN4.id} ]\n\n"
            )
            client_id.append(AYIIN4.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN5:
        try:
            await AYIIN5.start()
            clients.append(5)
            await ayiin_client(AYIIN5)
            me = await AYIIN5.get_me()
            AYIIN5.id = me.id
            AYIIN5.mention = me.mention
            AYIIN5.username = me.username
            if me.last_name:
                AYIIN5.name = me.first_name + " " + me.last_name
            else:
                AYIIN5.name = me.first_name
            #AYIIN5.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN5 in {AYIIN5.name} | [ {AYIIN5.id} ]\n\n"
            )
            client_id.append(AYIIN5.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN6:
        try:
            await AYIIN6.start()
            clients.append(6)
            await ayiin_client(AYIIN6)
            me = await AYIIN6.get_me()
            AYIIN6.id = me.id
            AYIIN6.mention = me.mention
            AYIIN6.username = me.username
            if me.last_name:
                AYIIN6.name = me.first_name + " " + me.last_name
            else:
                AYIIN6.name = me.first_name
            #AYIIN1.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN6 in {AYIIN6.name} | [ {AYIIN6.id} ]\n\n"
            )
            client_id.append(AYIIN6.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN7:
        try:
            await AYIIN7.start()
            clients.append(7)
            await ayiin_client(AYIIN7)
            me = await AYIIN7.get_me()
            AYIIN7.id = me.id
            AYIIN7.mention = me.mention
            AYIIN7.username = me.username
            if me.last_name:
                AYIIN7.name = me.first_name + " " + me.last_name
            else:
                AYIIN7.name = me.first_name
            #AYIIN7.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN7 in {AYIIN7.name} | [ {AYIIN7.id} ]\n\n"
            )
            client_id.append(AYIIN7.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN8:
        try:
            await AYIIN8.start()
            clients.append(8)
            await ayiin_client(AYIIN8)
            me = await AYIIN8.get_me()
            AYIIN8.id = me.id
            AYIIN8.mention = me.mention
            AYIIN8.username = me.username
            if me.last_name:
                AYIIN8.name = me.first_name + " " + me.last_name
            else:
                AYIIN8.name = me.first_name
            #AYIIN8.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN8 in {AYIIN8.name} | [ {AYIIN8.id} ]\n\n"
            )
            client_id.append(AYIIN8.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN9:
        try:
            await AYIIN9.start()
            clients.append(9)
            await ayiin_client(AYIIN9)
            me = await AYIIN9.get_me()
            AYIIN9.id = me.id
            AYIIN9.mention = me.mention
            AYIIN9.username = me.username
            if me.last_name:
                AYIIN9.name = me.first_name + " " + me.last_name
            else:
                AYIIN9.name = me.first_name
            #AYIIN9.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN9 in {AYIIN9.name} | [ {AYIIN9.id} ]\n\n"
            )
            client_id.append(AYIIN9.id)
        except Exception as ex:
            logs.info(str(ex))
    if AYIIN10:
        try:
            await AYIIN10.start()
            clients.append(10)
            await ayiin_client(AYIIN10)
            me = await AYIIN10.get_me()
            AYIIN10.id = me.id
            AYIIN10.mention = me.mention
            AYIIN10.username = me.username
            if me.last_name:
                AYIIN10.name = me.first_name + " " + me.last_name
            else:
                AYIIN10.name = me.first_name
            #AYIIN10.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN10 in {AYIIN10.name} | [ {AYIIN10.id} ]\n\n"
            )
            client_id.append(AYIIN10.id)
        except Exception as ex:
            logs.info(str(ex))
    logs.info(f"Connecting Database To {adB.name}")
    if adB.ping():
        logs.info(f"Succesfully Connect On {adB.name}")
    logs.info(
        f"Connect On [ {HOSTED_ON} ]\n"
    )



import asyncio
from os import path

from pyAyiin.exceptions import FFmpegReturnCodeError


class Convert(object):
    async def convert(file_path: str) -> str:
        out = path.join("raw_files", path.basename(file_path + ".raw"))
        if path.isfile(out):
            return out
        proc = await asyncio.create_subprocess_shell(
            cmd=(
                "ffmpeg "
                "-y -i "
                f"{file_path} "
                "-f s16le "
                "-ac 1 "
                "-ar 48000 "
                "-acodec pcm_s16le "
                f"{out}"
            ),
            stdin=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        await proc.communicate()
        if proc.returncode != 0:
            raise FFmpegReturnCodeError("FFmpeg did not return 0")
        return out



import logging

from telethon.tl.functions.channels import JoinChannelRequest as Get
from telethon.tl.types import MessageEntityMentionName

from ._wrappers import eod

LOGS = logging.getLogger(__name__)


try:
    import pybase64
except ImportError:
    pybase64 = None
    LOGS.info(
        "'pybase64' not found!\nInstall pybase64 to use This Function.."
    )


class Events(object):
    async def get_user_from_event(
        self,
        event,
        yinsevent=None,
        secondgroup=None,
        nogroup=False,
        noedits=False
    ):
        if yinsevent is None:
            yinsevent = event
        if nogroup is False:
            if secondgroup:
                args = event.pattern_match.group(2).split(" ", 1)
            else:
                args = event.pattern_match.group(1).split(" ", 1)
        extra = None
        try:
            if args:
                user = args[0]
                if len(args) > 1:
                    extra = "".join(args[1:])
                if user.isnumeric() or (user.startswith("-")
                                        and user[1:].isnumeric()):
                    user = int(user)
                if event.message.entities:
                    probable_user_mention_entity = event.message.entities[0]
                    if isinstance(
                            probable_user_mention_entity,
                            MessageEntityMentionName):
                        user_id = probable_user_mention_entity.user_id
                        user_obj = await event.client.get_entity(user_id)
                        return user_obj, extra
                if isinstance(user, int) or user.startswith("@"):
                    user_obj = await event.client.get_entity(user)
                    return user_obj, extra
        except Exception as e:
            LOGS.error(str(e))
        try:
            if nogroup is False:
                if secondgroup:
                    extra = event.pattern_match.group(2)
                else:
                    extra = event.pattern_match.group(1)
            if event.is_private:
                user_obj = await event.get_chat()
                return user_obj, extra
            if event.reply_to_msg_id:
                previous_message = await event.get_reply_message()
                if previous_message.sender_id is None:
                    if not noedits:
                        await eod(
                            yinsevent, "**ERROR: Dia adalah anonymous admin!**", 60
                        )
                    return None, None
                user_obj = await event.client.get_entity(previous_message.sender_id)
                return user_obj, extra
            if not args:
                if not noedits:
                    await eod(
                        yinsevent,
                        "**Mohon Reply Pesan atau Berikan User ID/Username pengguna!**",
                        60,
                    )
                return None, None
        except Exception as e:
            LOGS.error(str(e))
        if not noedits:
            await eod(
                yinsevent,
                "**Mohon Reply Pesan atau Berikan User ID/Username pengguna!**",
                60,
            )
        return None, None


    async def checking(self, client):
        gocheck = str(pybase64.b64decode("QEF5aWluU3VwcG9ydA=="))[2:15]
        checker = str(pybase64.b64decode("QEF5aWluQ2hhdA=="))[2:12]
        checkxd = str(pybase64.b64decode("QFN0b3J5QXlpaW4="))[2:13]
        checksbt = str(pybase64.b64decode("QFNoYXJpbmdVc2VyYm90"))[2:17]
        if client:
            try:
                await client(Get(gocheck))
                await client(Get(checker))
                await client(Get(checkxd))
                await client(Get(checksbt))
            except BaseException:
                pass



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

from ._database import AyiinDB
from .converter import Convert
from .helpers import Helpers
from .hosting import where_hosted
from .queue import Queue


class Methods(
    Convert,
    Helpers,
    Queue,
):
    pass
